@@ -16,24 +16,24 @@ import contextlib
 import logging
 import os
 import tempfile
-from typing import List, Union
+from tempfile import NamedTemporaryFile
+from typing import List, Optional, Tuple, Union
 
 import numpy
 import onnx
 
+from deepsparse.utils.extractor import Extractor
+
 
 try:
-    from sparsezoo import Zoo
-    from sparsezoo.objects import File, Model
+    from sparsezoo import File, Model
 
     sparsezoo_import_error = None
 except Exception as sparsezoo_err:
-    Zoo = None
     Model = object
     File = object
     sparsezoo_import_error = sparsezoo_err
 
-
 __all__ = [
     "ONNX_TENSOR_TYPE_MAP",
     "model_to_path",
@@ -44,6 +44,8 @@ __all__ = [
     "generate_random_inputs",
     "override_onnx_batch_size",
     "override_onnx_input_shapes",
+    "truncate_onnx_model",
+    "truncate_onnx_embedding_model",
 ]
 
 _LOGGER = logging.getLogger(__name__)
@@ -81,7 +83,12 @@ def model_to_path(model: Union[str, Model, File]) -> str:
     """
     Deals with the various forms a model can take. Either an ONNX file,
     a SparseZoo model stub prefixed by 'zoo:', a SparseZoo Model object,
-    or a SparseZoo ONNX File object that defines the neural network
+    or a SparseZoo ONNX File object that defines the neural network. Noting
+    the model will be downloaded automatically if a SparseZoo stub is passed
+
+    :param model: Either a local str path or SparseZoo stub to the model. Can
+        also be a sparsezoo.Model or sparsezoo.File object
+    :returns: The absolute local str path to the model
     """
     if not model:
         raise ValueError("model must be a path, sparsezoo.Model, or sparsezoo.File")
@@ -90,14 +97,14 @@ def model_to_path(model: Union[str, Model, File]) -> str:
         # load SparseZoo Model from stub
         if sparsezoo_import_error is not None:
             raise sparsezoo_import_error
-        model = Zoo.load_model_from_stub(model)
+        model = Model(model)
 
     if Model is not object and isinstance(model, Model):
         # default to the main onnx file for the model
-        model = model.onnx_file.downloaded_path()
+        model = model.onnx_model.path
     elif File is not object and isinstance(model, File):
         # get the downloaded_path -- will auto download if not on local system
-        model = model.downloaded_path()
+        model = model.path
 
     if not isinstance(model, str):
         raise ValueError("unsupported type for model: {}".format(type(model)))
@@ -139,7 +146,7 @@ def get_input_names(onnx_filepath: str) -> List[str]:
     :param onnx_filepath: File path to ONNX model
     :return: List of string names
     """
-    return [input.name for input in get_external_inputs(onnx_filepath)]
+    return [input_.name for input_ in get_external_inputs(onnx_filepath)]
 
 
 def get_output_names(onnx_filepath: str) -> List[str]:
@@ -264,3 +271,141 @@ def override_onnx_input_shapes(
     finally:
         os.unlink(shaped_model.name)
         shaped_model.close()
+
+
+def truncate_onnx_model(
+    onnx_filepath: str,
+    output_filepath: str,
+    final_node_names: List[str],
+    graph_output_names: List[str],
+    graph_output_shapes: Optional[List[List[int]]] = None,
+) -> None:
+    """
+    :param onnx_filepath: file path to onnx model
+    :param output_filepath: file path to save new onnx model
+    :param final_node_names: list of node names whose outputs will become the
+        outputs of the graph
+    :param graph_output_names: list of names to call the graph outputs. Names
+        correspond with the outputs specified in final_node_names
+    :param graph_output_shapes: list of shapes for each output. If not provided,
+        defaults to [None] for each output and leads to slight performance loss
+    :return: None
+    """
+    if graph_output_shapes is None:
+        graph_output_shapes = [None] * len(final_node_names)
+
+    if len(final_node_names) != len(graph_output_names) != len(graph_output_shapes):
+        raise ValueError(
+            f"length of final_node_names {len(final_node_names)}, "
+            f"graph_output_names {len(graph_output_names)}, and "
+            f"graph_output_shapes {len(graph_output_shapes)} must all match"
+        )
+
+    if len(set(final_node_names)) != len(final_node_names):
+        raise ValueError("final_node_names must not contain duplicate names")
+
+    if len(set(graph_output_names)) != len(graph_output_names):
+        raise ValueError("graph_output_names must not contain duplicate names")
+
+    model = onnx.load(onnx_filepath)
+    final_nodes = [node for node in model.graph.node if node.name in final_node_names]
+
+    if len(final_nodes) != len(final_node_names):
+        raise ValueError("Could not find final node names in model graph")
+
+    for final_node, graph_output_name, graph_output_shape in zip(
+        final_nodes, graph_output_names, graph_output_shapes
+    ):
+        # write each node's output to new output
+        [final_node.output.pop() for _ in final_node.output]
+        final_node.output.append(graph_output_name)
+
+        # write graph output. TODO: use ort to find real shapes and types
+        output_value_info = onnx.helper.make_tensor_value_info(
+            graph_output_name, onnx.TensorProto.UNDEFINED, graph_output_shape
+        )
+        model.graph.output.append(output_value_info)
+
+    # collect graph inputs
+    graph_input_names = [input.name for input in model.graph.input]
+
+    # use extractor to create and write subgraph
+    original_num_nodes = len(model.graph.node)
+    extractor = Extractor(model)
+    extracted_model = extractor.extract_model(
+        input_names=graph_input_names, output_names=graph_output_names
+    )
+    extracted_num_nodes = len(extracted_model.graph.node)
+    _LOGGER.info(
+        f"Truncating model graph to {final_node_names}. "
+        f"Removed {original_num_nodes - extracted_num_nodes} nodes, "
+        f"{extracted_num_nodes} remaining"
+    )
+
+    for output in extracted_model.graph.output:
+        if len(output.type.tensor_type.shape.dim) == 0:
+            # ONNX checker treats None shapes and empty shapes
+            # differently, clear None shape to pass checker
+            output.type.tensor_type.shape.Clear()
+
+    # save and check model
+    onnx.save(extracted_model, output_filepath)
+    onnx.checker.check_model(output_filepath)
+
+
+def truncate_onnx_embedding_model(
+    model_path: str,
+    emb_extraction_layer: Union[int, str, None] = None,
+    output_filepath: Optional[str] = None,
+) -> Tuple[str, Optional[NamedTemporaryFile]]:
+    """
+     :param model_path: path of onnx file to be cut
+    :param emb_extraction_layer: if an int, last layer to include. If a
+        string, then the name of the last node in the truncated graph.
+        default is None.
+    :param output_filepath: path to write resulting onnx file. If not provided,
+        will create a temporary file path that will be destroyed on program end
+    :return: if no output path, a tuple of the saved path to the model, list of
+        model output names, and reference to the tempfile object will be returned
+        otherwise, a tuple containing the given output_path argument, the model
+        output names, and None
+    """
+
+    tmp_file = None
+    if output_filepath is None:
+        tmp_file = NamedTemporaryFile()
+        output_filepath = tmp_file.name
+
+    # determine where to cut the model
+    model = onnx.load(model_path)
+    if isinstance(emb_extraction_layer, str):
+        final_node = None
+        for graph_node in model.graph.node:
+            if graph_node.name == emb_extraction_layer:
+                final_node = graph_node
+
+        if final_node is None:
+            raise RuntimeError(
+                f"Unable to find node {emb_extraction_layer} for extraction in graph"
+            )
+
+        final_node_name = final_node.name
+        graph_output_name = final_node.output[0]
+    else:
+        final_node_name = model.graph.node[emb_extraction_layer].name
+        graph_output_name = model.graph.node[emb_extraction_layer].output[0]
+
+        if final_node_name is None:
+            raise ValueError(
+                f"Node at index {emb_extraction_layer} does not have a name set"
+            )
+
+    truncate_onnx_model(
+        onnx_filepath=model_path,
+        output_filepath=output_filepath,
+        final_node_names=[final_node_name],
+        graph_output_names=[graph_output_name],
+        graph_output_shapes=None,
+    )
+
+    return output_filepath, tmp_file
